@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2014 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2018 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -33,14 +33,14 @@ contains
 !
 !----------------------------------------------------
 subroutine adjust_data_codeunits
- use system_utils,  only:renvironment,envlist,ienvironment,lenvironment
- use labels,        only:ih,ix,ivx,label,get_sink_type,ipmass,idustfrac,irho
- use settings_data, only:ncolumns,ndimV,icoords,ndim,debugmode,ntypes,iverbose,fakedust
- use particle_data, only:dat,npartoftype,iamtype
- use geometry,      only:labelcoord
- use filenames,     only:ifileopen,nstepsinfile
- use part_utils,    only:locate_first_two_of_type,locate_nth_particle_of_type,get_binary
- implicit none
+ use system_utils,    only:renvironment,envlist,ienvironment,lenvironment
+ use labels,          only:ih,ix,ivx,label,get_sink_type,ipmass,idustfrac,irho
+ use settings_data,   only:ncolumns,ndimV,icoords,ndim,debugmode,ntypes,iverbose,fakedust
+ use particle_data,   only:dat,npartoftype,iamtype
+ use geometry,        only:labelcoord
+ use filenames,       only:ifileopen,nstepsinfile
+ use part_utils,      only:locate_first_two_of_type,locate_nth_particle_of_type,get_binary
+ use settings_render, only:ifastrender
  real :: hmin,dphi
  real, dimension(3) :: vsink,xyzsink,x0,v0
  character(len=20), dimension(3) :: list
@@ -62,6 +62,8 @@ subroutine adjust_data_codeunits
        where (dat(:,ih,:) < hmin .and. dat(:,ih,:).gt.0.)
           dat(:,ih,:) = hmin
        end where
+       print "(a)",' >> Switching accelerated rendering ON'
+       ifastrender = .true.
     endif
  endif
 
@@ -144,7 +146,7 @@ subroutine adjust_data_codeunits
           else
              if (iverbose.ge.1) print*
              if (isink.lt.10) then
-                print "(a,i1,a)",' :: CENTREING ON SINK ',isink,' from SPLASH_CENTRE_ON_SINK setting'          
+                print "(a,i1,a)",' :: CENTREING ON SINK ',isink,' from SPLASH_CENTRE_ON_SINK setting'
              else
                 print "(a,i3,a)",' :: CENTREING ON SINK ',isink,' from SPLASH_CENTRE_ON_SINK setting'
              endif
@@ -237,7 +239,7 @@ pure subroutine shift_particles(dat,np,ndim,ndimV,ncol,x0,v0)
 
  call shift_positions(dat,np,ndim,x0)
  call shift_velocities(dat,np,ndimV,ncol,v0)
- 
+
 end subroutine shift_particles
 
 !------------------------------------------------------
@@ -254,7 +256,7 @@ pure subroutine shift_positions(dat,np,ndim,x0)
  do icol=1,ndim
     dat(1:np,ix(icol)) = dat(1:np,ix(icol)) - x0(icol)
  enddo
- 
+
 end subroutine shift_positions
 
 !------------------------------------------------------
@@ -288,7 +290,7 @@ subroutine fake_twofluids(istart,iend,ndim,ndimV,dat,npartoftype,iamtype)
                           irho,ix,ih,ipmass,ivx,ideltav,ideltavsum
  use mem_allocation, only:alloc
  use particle_data,  only:maxpart,maxstep,maxcol
- use settings_data,  only:iverbose,required,ndusttypes,idustfrac_plot,ideltav_plot
+ use settings_data,  only:iverbose,ndusttypes,idustfrac_plot,ideltav_plot
  integer,                       intent(in)    :: istart,iend,ndim,ndimV
  real,    dimension(:,:,:),     intent(inout), allocatable :: dat
  integer, dimension(:,:),       intent(inout), allocatable :: npartoftype
@@ -330,7 +332,7 @@ subroutine fake_twofluids(istart,iend,ndim,ndimV,dat,npartoftype,iamtype)
        ndust = 0
        !--zero the properties of newly created dust particles
        dat(ntoti+1:ntoti+npartoftype(1,i),:,i) = 0.
-       if (idustfrac_temp > size(dat(1,:,1))) then
+       if (idustfrac_temp > size(dat(1,:,1)) .or. idustfrac_temp <= 0) then
           print*,' ERROR: idustfrac out of range: cannot create fake dust particles'
           return
        endif
@@ -357,10 +359,21 @@ subroutine fake_twofluids(istart,iend,ndim,ndimV,dat,npartoftype,iamtype)
              if (ih.gt.0)   dat(jdust,ih,i)         = dat(j,ih,i)
              if (irho.gt.0) dat(jdust,irho,i)       = rhodust
              if (idustfracsum == 0) then
+                !--copy the dustfracion
                 dat(jdust,idustfrac,i) = dustfraci
+                if (ideltav /= 0) then
+                   !--copy the deltav's
+                   dat(jdust,ideltav:ideltav+ndimV-1,i) = dat(j,ideltav:ideltav+ndimV-1,i)
+                endif
              else
+                !--copy the dustfracion
                 dat(jdust,idustfracsum:idustfracsum+ndusttypes,i) = &
                     dat(j,idustfracsum:idustfracsum+ndusttypes,i)
+                !--copy the deltav's
+                if (ideltavsum /= 0) then
+                   dat(jdust,ideltavsum:ideltavsum+ndimV*(ndusttypes+1)-1,i) = &
+                              dat(j,ideltavsum:ideltavsum+ndimV*(ndusttypes+1)-1,i)
+                endif
              endif
              iamtype(ntoti + ndust,i) = 2
 
