@@ -27,7 +27,7 @@
 module colours
  implicit none
  integer, parameter :: ncolourmax = 256
- integer, parameter :: ncolourschemes = 39
+ integer, parameter :: ncolourschemes = 40
  character(len=17), dimension(ncolourschemes), parameter :: schemename = &
     (/'greyscale        ', &
       'red              ', &
@@ -67,13 +67,17 @@ module colours
       'viridis          ', &
       'ocean            ', &
       'casa blue        ', &
-      'green-brown      '/)
+      'green-brown      ', &
+      'Doppler shift    '/)
+  integer, parameter :: icustom = 100 ! number 100 is custom colour table
 !
 !--rgb colours of the colour table are stored in the array below
 !  this is used for colour blending (opacity rendering)
 !
-  integer :: ifirstcolour, ncolours
+  integer :: ifirstcolour, ncolours,ncoltable
   real, dimension(3,ncolourmax) :: rgbtable
+  real, dimension(ncolourmax)   :: ltable
+  integer, private :: icolourscheme_prev = 0
 
 contains
 
@@ -82,9 +86,8 @@ contains
 !      ** add your own here **
 ! ------------------------------------------------------------------------
 subroutine colour_set(icolourscheme)
-  use plotlib, only:plot_qcol,plot_qcir,plot_scir,plot_ctab,plot_scr,plot_qcr
+  use plotlib,       only:plot_qcol,plot_qcir,plot_scir,plot_ctab,plot_scr,plot_qcr
   use settings_data, only:debugmode
-  implicit none
   integer, intent(in) :: icolourscheme
   integer :: i,icolmin,icolmax,ncolmax,nset,index
   real :: brightness,contrast
@@ -118,12 +121,12 @@ subroutine colour_set(icolourscheme)
   if (debugmode) print*,'DEBUG: querying colour index range'
   call plot_scir(ifirstcolour,ifirstcolour+ncolours)
 
-  if (abs(icolourscheme).le.ncolourschemes) then
-     brightness = 0.5
-     contrast = 1.0
-     !--invert colour table for negative values
-     if (icolourscheme.lt.0) contrast = -1.0
+  brightness = 0.5
+  contrast = 1.0
+  !--invert colour table for negative values
+  if (icolourscheme.lt.0) contrast = -1.0
 
+  if (abs(icolourscheme).le.ncolourschemes) then
      select case(abs(icolourscheme))
      case(1)
      !--greyscale
@@ -507,6 +510,13 @@ subroutine colour_set(icolourscheme)
      redarr(1:nset) =   (/0.0,0.66,0.5/)
      greenarr(1:nset) = (/0.83,0.66,0.33/)
      bluearr(1:nset) =  (/0.33,0.66,0.17/)
+     case(40)
+     !--Red-Blue
+     nset = 11
+     lumarr(1:nset)  = (/0.00,0.10,0.20,0.30,0.40,0.50,0.60,0.70,0.80,0.90,1.00/)
+     redarr(1:nset)  = (/0.40,0.69,0.84,0.95,0.99,0.97,0.82,0.58,0.27,0.14,0.02/)
+     greenarr(1:nset)= (/0.00,0.09,0.38,0.63,0.86,0.96,0.90,0.77,0.58,0.41,0.20/)
+     bluearr(1:nset) = (/0.12,0.16,0.30,0.49,0.78,0.96,0.94,0.87,0.77,0.68,0.39/)
      end select
 
      if (debugmode) print*,'DEBUG: setting colour table'
@@ -517,13 +527,25 @@ subroutine colour_set(icolourscheme)
 !--if icolourscheme = ncolourschemes+1 set the PGPLOT colour indices
 !  from the contents of the rgbtable array
 !
-  if (abs(icolourscheme).eq.ncolourschemes+1) then
-     call plot_scir(ifirstcolour,ifirstcolour+ncolourmax)
-     do i=1,ncolourmax
+  if (abs(icolourscheme).eq.icustom) then
+     print "(1x,a)",'using colour scheme other'
+     !
+     !--for giza we have to call set_ctab directly with all 256 colours
+     !
+     do i=1,ncoltable
+        ltable(i) = (i-1)/real(ncoltable-1)
+     enddo
+     nset = ncoltable
+     call plot_ctab(ltable(1:nset),rgbtable(1,1:nset),rgbtable(2,1:nset),rgbtable(3,1:nset), &
+                    nset,contrast,brightness)
+
+     ! for pgplot we set the colours in the colour table individually
+     ncolours = ncoltable-1
+     call plot_scir(ifirstcolour,ifirstcolour+ncolours)
+     do i=1,ncolours
         index = ifirstcolour + (i-1)
         call plot_scr(index,rgbtable(1,i),rgbtable(2,i),rgbtable(3,i))
      enddo
-     print "(1x,a)",'using colour scheme other'
 
   elseif (abs(icolourscheme).le.ncolourschemes) then
 !
@@ -535,81 +557,20 @@ subroutine colour_set(icolourscheme)
         call plot_qcr(index,rgbtable(1,i),rgbtable(2,i),rgbtable(3,i))
      enddo
 
-     if (icolourscheme.lt.0) then
-        print "(1x,a)",'using colour scheme inverse '//trim(schemename(abs(icolourscheme)))
-     else
-        print "(1x,a)",'using colour scheme '//trim(schemename(icolourscheme))
+     if (icolourscheme /= icolourscheme_prev) then ! silent if not changed
+        if (icolourscheme.lt.0) then
+           print "(1x,a)",'using colour scheme inverse '//trim(schemename(abs(icolourscheme)))
+        else
+           print "(1x,a)",'using colour scheme '//trim(schemename(icolourscheme))
+        endif
      endif
+     icolourscheme_prev = icolourscheme
   else
      print "(1x,a)",'warning: unknown colour scheme - uses default greyscale'
 
   endif
   if (debugmode) print*,'DEBUG: finished colour_set'
 
-  return
-
 end subroutine colour_set
 
-!------------------------------------------------
-! demonstration plot of all the colour schemes
-!------------------------------------------------
-subroutine colour_demo
-  implicit none
-!  integer :: i,j,nc
-  !
-  !--npixx should be >= ncolours in setcolours.f
-  !
-!  integer, parameter :: npixx = ncolourmax
-!  integer, parameter :: npixy = npixx/10
-!  real, dimension(npixx,npixy) :: sample
-!  real :: xmin,xmax,ymin,ymax,dx,dy,trans(6)
-!  character(len=10) :: string
-
-!  call pgbegin(0,'?',1,ncolourschemes)
-!!  call pgpaper(6.0,8.0) !!!0.25/sqrt(2.))
-
-!  xmin = 0.0
-!  xmax = 1.0
-!  ymin = 0.0
-!  ymax = 0.1
-!  dx = (xmax-xmin)/float(npixx)
-!  dy = (ymax-ymin)/float(npixy)
-!  trans(1) = xmin - 0.5*dx
-!  trans(2) = dx
-!  trans(3) = 0.0
-!  trans(4) = ymin - 0.5*dy
-!  trans(5) = 0.0
-!  trans(6) = dy
-
-!  do j=1,npixy
-!     do i=1,npixx
-!        sample(i,j) = (i-1)*dx
-!     enddo
-!  enddo
-!!  call pgsch(2.0)
-!!  call pgenv(xmin,xmax,ymin,ymax,0,-1)
-!!  call pgsch(1.0)
-!!  call pggray(sample,npixx,npixy,1,npixx,1,npixy, &
-!!              minval(sample),maxval(sample),trans)
-!!  call pgnumb(1,0,0,string,nc)
-!!  call pgsch(7.0)
-!!  call pgmtxt('t',0.5,0.5,0.5,string(1:nc)//': '//trim(schemename(1)))
-
-!  do i=1,ncolourschemes
-!     call pgsch(2.0)
-!     call pgenv(xmin,xmax,ymin,ymax,0,-1)
-!     call pgsch(7.0)
-!     call pgnumb(i,0,0,string,nc)
-!     call pgmtxt('t',0.5,0.5,0.5,string(1:nc)//': '//trim(schemename(i)))
-!     call colour_set(i)
-!     call pgimag(sample,npixx,npixy,1,npixx,1,npixy, &
-!                 minval(sample),maxval(sample),trans)
-!  enddo
-
-!  call pgsch(1.0)
-!  call pgend
-
-end subroutine colour_demo
-
-
 end module colours
