@@ -15,7 +15,7 @@
 !  a) You must cause the modified files to carry prominent notices
 !     stating that you changed the files and the date of any change.
 !
-!  Copyright (C) 2005-2014 Daniel Price. All rights reserved.
+!  Copyright (C) 2005-2015 Daniel Price. All rights reserved.
 !  Contact: daniel.price@monash.edu
 !
 !-----------------------------------------------------------------
@@ -61,7 +61,7 @@ subroutine exact_sedov(iplot,time,gam,rhozero,energy,rmax,rplot,yplot,ierr)
 
   npts = size(rplot)
   eta_0  = eta0(gam,ndim)
-  print*,' eta0 = ',eta_0
+  print*,' eta0 = ',eta_0, ' gamma = ',gam
 
   power = 1./(ndim+2)
   dr = rmax/float(npts-1)
