@@ -65,7 +65,9 @@
 ! in the module 'particle_data'
 !-------------------------------------------------------------------------
 module asciiread
+ use labels, only:lenlabel,label
  integer :: icoltype
+ character(len=lenlabel), dimension(size(label)) :: label_orig
 
 end module asciiread
 
@@ -76,21 +78,22 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
   use mem_allocation, only:alloc
   use asciiutils,     only:get_ncolumns,get_column_labels,isdigit
   use system_utils,   only:ienvironment,renvironment
-  use asciiread,      only:icoltype
+  use asciiread,      only:icoltype,label_orig
   use labels,         only:lenlabel,labeltype,print_types,label
   implicit none
   integer, intent(in)          :: indexstart,ipos
   integer, intent(out)         :: nstepsread
   character(len=*), intent(in) :: rootname
   integer :: i,j,ierr,iunit,ncolstep,ncolenv,nerr,iheader_time,iheader_gamma
-  integer :: nprint,npart_max,nstep_max,icol,nheaderlines,nheaderenv,itype,nlabels
-  integer :: noftype(maxparttypes),iverbose_was
+  integer :: nprint,npart_max,nstep_max,nheaderlines,nheaderenv,itype,nlabels
+  integer :: noftype(maxparttypes),iverbose_was,imethod
   logical :: iexist,timeset,gammaset,got_labels
   real    :: dummyreal
+  real, allocatable :: dattemp(:)
   character(len=len(rootname)+4) :: dumpfile
   character(len=2048)  :: line
   character(len=lenlabel), dimension(size(label)) :: tmplabel
-  character(len=30)   :: string
+  character(len=10)  :: str,strc
   integer, parameter :: notset = -66
 
   nstepsread = 0
@@ -100,7 +103,7 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
 
   dumpfile = trim(rootname)
 
-  if (iverbose.gt.0) print "(1x,a)",'reading ascii format'
+  if (iverbose.gt.1) print "(1x,a)",'reading ascii format'
   print "(26('>'),1x,a,1x,26('<'))",trim(dumpfile)
   !
   !--check if first data file exists
@@ -120,7 +123,7 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
   nstepsread = 0
   icoltype = 0       ! no particle type defined by default
   got_labels = .false.
-
+  label_orig = ''
   !
   !--open the file and read the number of particles
   !
@@ -150,10 +153,14 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
      do i=1,nheaderlines
         !--search through header for column labels
         read(iunit,"(a)",iostat=ierr) line
-        if ((i.eq.1 .or. .not.got_labels) .and. ncolstep > 1) then
-           call get_column_labels(trim(line),nlabels,tmplabel)
-           if (nlabels>=ncolstep .and. .not.(isdigit(tmplabel(1)(1:1)) .or. tmplabel(1)(1:1)=='.')) then
-              label(1:ncolstep) = tmplabel(1:ncolstep)
+        !--if we get nlabels > ncolumns, use them, but keep trying for a better match
+        if ((.not.got_labels .or. nlabels /= ncolstep) .and. ncolstep > 1 ) then
+           call get_column_labels(trim(line),nlabels,tmplabel,method=imethod)
+           ! use labels if > ncolumns, but replace if we match exact number on subsequent line
+           if ((got_labels .and. nlabels == ncolstep) .or. &
+               (.not.got_labels .and. nlabels >= ncolstep  & ! only allow single-spaced labels if == ncols
+                .and. (.not.(imethod>=4).or.nlabels==ncolstep))) then
+              label_orig(1:ncolstep) = tmplabel(1:ncolstep)
               got_labels = .true.
            endif
         endif
@@ -184,7 +191,6 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
   if (j.gt.maxstep) then
      call alloc(maxpart,j+1,maxcol,mixedtypes=(icoltype > 0))
   endif
-
 !
 !--can set either set the time and gamma explicitly
 !  using environment variables (fixed for all files)
@@ -207,7 +213,7 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
 !
 !--read header lines, try to use it to set time
 !
-  if (nheaderlines.gt.0 .and. iverbose.gt.0) print*,'skipping ',nheaderlines,' header lines'
+  !if (nheaderlines.gt.0 .and. iverbose.gt.0) print*,'skipping ',nheaderlines,' header lines'
   do i=1,nheaderlines
      !--read header lines as character strings
      !  so that blank lines are counted in nheaderlines
@@ -249,6 +255,10 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
      endif
   enddo
 !
+! allocate temporary array to read each line
+!
+  allocate(dattemp(ncolstep))
+!
 !--now read the timestep data in the dumpfile
 !
   i = 0
@@ -260,9 +270,10 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
      i = i + 1
      if (i.gt.npart_max) then ! reallocate memory if necessary
         npart_max = 10*npart_max
-        call alloc(npart_max,nstep_max,ncolstep+ncalc,mixedtypes=(icoltype > 0))        
+        call alloc(npart_max,nstep_max,ncolstep+ncalc,mixedtypes=(icoltype > 0))
      endif
-     read(iunit,*,iostat=ierr) (dat(i,icol,j),icol = 1,ncolstep)
+     read(iunit,*,iostat=ierr) dattemp(1:ncolstep)
+     dat(i,1:ncolstep,j) = dattemp(1:ncolstep)
      if (icoltype > 0 .and. icoltype <= ncolstep .and. ierr==0 .and. (size(iamtype(:,j)) > 1)) then
         !--set particle type from type column
         itype = nint(dat(i,icoltype,j))
@@ -281,6 +292,7 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
         i = i - 1 ! ignore lines with errors
      endif
   enddo overparts
+  if (allocated(dattemp)) deallocate(dattemp)
 
   nprint = i - 1
   nstepsread = nstepsread + 1
@@ -288,8 +300,20 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
   if (nerr > 10) then
      print "(a,i8,a)",' *** WARNING: errors whilst reading file on ',nerr,' lines: skipped these ***'
   endif
-  if (ierr < 0) then
-     print "(2(a,i10))",' read npts = ',nprint,' ncolumns = ',ncolstep
+  if (ierr < 0 .and. (icoltype <=0 .or. icoltype > ncolstep)) then
+     write(str,"(i10)") nprint
+     str = adjustl(str)
+     write(strc,"(i10)") ncolstep
+     strc = adjustl(strc)
+     if (nheaderlines.gt.0 .and. iverbose.gt.0) then
+        if (nheaderlines > 10) then
+           print "(a,i3,a)",' npts = '//trim(str)//', ncols = '//trim(strc)//', skipped ',nheaderlines,' header lines'
+        else
+           print "(a,i1,a)",' npts = '//trim(str)//', ncols = '//trim(strc)//', skipped ',nheaderlines,' header lines'
+        endif
+     else
+        print "(a)",' npts = '//trim(str)//', ncols = '//trim(strc)
+     endif
   endif
 
   npartoftype(:,j) = 0
@@ -302,33 +326,31 @@ subroutine read_data(rootname,indexstart,ipos,nstepsread)
 
   close(iunit)
 
-return
 end subroutine read_data
 
-!!-------------------------------------------------------------------
-!! set labels for each column of data
-!!
-!! read these from a file called 'columns' in the current directory
-!! then take sensible guesses as to which quantities are which
-!! from the column labels
-!!
-!!-------------------------------------------------------------------
-
+!-------------------------------------------------------------------
+! set labels for each column of data
+!
+! read these from a file called 'columns' in the current directory
+! then take sensible guesses as to which quantities are which
+! from the column labels
+!
+!-------------------------------------------------------------------
 subroutine set_labels
-  use asciiutils,      only:lcase
+  use asciiutils,      only:lcase,match_taglist,find_repeated_tags
   use labels,          only:label,labeltype,ix,irho,ipmass,ih,iutherm, &
-                            ipr,ivx,iBfirst,iamvec,labelvec,lenlabel
-  !use params,          only:maxparttypes
+                            ipr,ivx,iBfirst,iamvec,labelvec,lenlabel, &
+                            make_vector_label
   use settings_data,   only:ncolumns,ndim,ndimV,UseTypeInRenderings,iverbose
   use geometry,        only:labelcoord
   use system_commands, only:get_environment
   use filenames,       only:fileprefix
-  use asciiread,       only:icoltype
+  use asciiread,       only:icoltype,label_orig
   implicit none
   integer                 :: i,ierr,ndimVtemp
   character(len=120)      :: columnfile
   character(len=lenlabel) :: labeli
-  logical                 :: iexist
+  logical                 :: iexist,got_time
 !
 !--read column labels from the columns file if it exists
 !
@@ -362,14 +384,15 @@ subroutine set_labels
 
   open(unit=51,file=trim(columnfile),status='old',iostat=ierr)
   if (ierr /=0) then
-     if (iverbose > 0) then
-        print "(3(/,a))",' WARNING: columns file not found: using default labels',&
+!     print*,'HERE ',label(1)
+     if (iverbose > 0 .and. len_trim(label_orig(1))==0) then
+        print "(3(/,a))",' WARNING: column labels not found in file header:',&
                          ' To change the labels, create a file called ''columns'' ',&
                          '  in the current directory with one label per line'
      endif
   else
      overcols: do i=1,ncolumns
-        read(51,"(a)",iostat=ierr) label(i)
+        read(51,"(a)",iostat=ierr) label_orig(i)
         if (ierr < 0) then
            if (iverbose > 0) print "(a,i3)",' end of file in columns file: read to column ',i-1
            exit overcols
@@ -380,26 +403,49 @@ subroutine set_labels
      enddo overcols
      close(unit=51)
   endif
-  
+!
+!--re-copy the labels from their originals
+!  this is to avoid trying to match tags on labels
+!  which have already been modified by splash (e.g. vectors)
+!
+  do i=1,ncolumns
+     if (len_trim(label_orig(i)) > 0) label(i) = trim(label_orig(i))
+  enddo
+!
+!--first, look for 'x','y','z' as consecutive labels
+!  to determine the number of dimensions
+!
+  call match_taglist((/'x','y','z'/),label(1:ncolumns),ix(1),ndim)
+  do i=2,ndim
+     ix(i) = ix(1)+i-1
+  enddo
+  call match_taglist((/'vx','vy','vz'/),lcase(label(1:ncolumns)),ivx,ndimV)
+  call match_taglist((/'bx','by','bz'/),lcase(label(1:ncolumns)),iBfirst,ndimVtemp)
+  if (ndimV==0 .and. ivx==0) call match_taglist((/'ux','uy','uz'/),lcase(label(1:ncolumns)),ivx,ndimV)
+
+  got_time = .false.
   do i=1,ncolumns
 !
 !--compare all strings in lower case, trimmed and with no preceding spaces
 !
      labeli = trim(adjustl(lcase(label(i))))
+     if (trim(labeli)=='t' .or. trim(labeli)=='time') got_time = .true.
 !
 !--guess positions of various quantities from the column labels
 !
-     if (ndim.le.0 .and. (labeli(1:1).eq.'x' .or. labeli(1:1).eq.'r')) then
-        ndim = 1
-        ix(1) = i
-     endif
-     if (ndim.eq.1 .and. i.eq.ix(1)+1 .and. (labeli(1:1).eq.'y' .or. labeli(1:1).eq.'z')) then
-        ndim = 2
-        ix(2) = i
-     endif
-     if (ndim.eq.2 .and. i.eq.ix(2)+1 .and. labeli(1:1).eq.'z') then
-        ndim = 3
-        ix(3) = i
+     if (.not.got_time) then
+        if (ndim.le.0 .and. (labeli(1:1).eq.'x' .or. trim(labeli).eq.'r' .or. labeli(1:3).eq.'rad')) then
+           ndim = 1
+           ix(1) = i
+        endif
+        if (ndim.eq.1 .and. i.eq.ix(1)+1 .and. (labeli(1:1).eq.'y' .or. labeli(1:1).eq.'z')) then
+           ndim = 2
+           ix(2) = i
+        endif
+        if (ndim.eq.2 .and. i.eq.ix(2)+1 .and. labeli(1:1).eq.'z') then
+           ndim = 3
+           ix(3) = i
+        endif
      endif
      if (labeli(1:3).eq.'den' .or. index(labeli,'rho').ne.0 .or. labeli(1:3).eq.'\gr' .or. &
          (index(labeli,'density').ne.0 .and. irho==0)) then
@@ -419,81 +465,65 @@ subroutine set_labels
      elseif (labeli(1:2).eq.'pr' .or. trim(labeli).eq.'p' .or. &
             (index(labeli,'pressure').ne.0 .and. ipr==0)) then
         ipr = i
-     elseif (ivx.eq.0 .and. labeli(1:1).eq.'v') then
-        ivx = i
-        ndimV = 1
      elseif (icoltype==0 .and. index(labeli,'type').ne.0) then
         icoltype = i
      endif
-     !--set ndimV as number of columns with v as label
-     if (ivx.gt.0 .and. i.gt.ivx .and. i.le.ivx+2) then
-        if (labeli(1:1).eq.'v') ndimV = i - ivx + 1
-     endif
-     if (iBfirst.eq.0 .and. (labeli(1:2).eq.'bx')) then
-        iBfirst = i
-     endif
-     !--set ndimV as number of columns with v as label
-     if (iBfirst.gt.0 .and. i.gt.iBfirst .and. i.le.iBfirst+2) then
-        if (labeli(1:1).eq.'b') then
-           ndimVtemp = i - iBfirst + 1
-           if (ndimV.gt.0 .and. ndimVtemp.gt.ndimV) then
-              if (iverbose > 0) print "(a)",' WARNING: possible confusion with vector dimensions'
-              ndimV = ndimVtemp
-           endif
-        endif
-     endif
   enddo
-
-  if (ndim.lt.1) ndimV = 0
+!
+!--try to find vectors by looking for multiple entries starting with 'v'
+!
+  if (ndim > 0 .and. ivx==0) call find_repeated_tags('v_',ncolumns,label,ivx,ndimV)
+  if (ndimVtemp.gt.ndimV .and. iverbose > 0) &
+     print "(a)",' WARNING: possible confusion with vector dimensions'
+!
+!--ignore label identifications if no spatial coordinates found
+!
+  if (ndim.lt.1) then
+     ndimV = 0
+     irho = 0
+     ipmass = 0
+     ih = 0
+     iutherm = 0
+     ipr = 0
+     ivx = 0
+  endif
   if (iverbose > 0) then
-     if (ndim.gt.0) print "(a,i1)",' Assuming number of dimensions = ',ndim
-     if (ndim.gt.0) print "(a,i2,a,i2)",' Assuming positions in columns ',ix(1),' to ',ix(ndim)
-
-     if (ndimV.gt.0) print "(a,i1)",' Assuming vectors have dimension = ',ndimV
-     if (irho.gt.0) print "(a,i2)",' Assuming density in column ',irho
-     if (ipmass.gt.0) print "(a,i2)",' Assuming particle mass in column ',ipmass
-     if (ih.gt.0) print "(a,i2)",' Assuming smoothing length in column ',ih
-     if (iutherm.gt.0) print "(a,i2)",' Assuming thermal energy in column ',iutherm
-     if (ipr.gt.0) print "(a,i2)",' Assuming pressure in column ',ipr
+     if (ndim.gt.0) print "(a,i1,a,i2,a,i2)",' Assuming ',ndim,' dimensions, coords in cols ',ix(1),' to ',ix(ndim)
+     !if (ndimV.gt.0) print "(a,i1)",' Assuming vectors have dimension = ',ndimV
+     if (ndim > 0 .and. (irho > 0 .or. ipmass>0 .or. ih > 0)) write(*,"(a)",advance='no') ' Assuming'
+     if (irho.gt.0) write(*,"(a,i2)",advance='no') ' density in column ',irho
+     if (ipmass.gt.0) write(*,"(a,i2)",advance='no') ', mass in ',ipmass
+     if (ih.gt.0) write(*,"(a,i2)",advance='no') ', h in ',ih
+     if (iutherm.gt.0) write(*,"(/,a,i2)") ' Assuming thermal energy in ',iutherm
+     if (iutherm==0 .and. ipr > 0) write(*,"(/,a)",advance='no') ' Assuming'
+     if (ipr.gt.0) write(*,"(a,i2)",advance='no') ' pressure in column ',ipr
      if (ivx.gt.0) then
+        if (ipr==0 .and. iutherm==0) write(*,*)
         if (ndimV.gt.1) then
-           print "(a,i2,a,i2)",' Assuming velocity in columns ',ivx,' to ',ivx+ndimV-1
+           print "(a,i2,a,i2)",' Assuming velocity in cols ',ivx,' to ',ivx+ndimV-1
         else
            print "(a,i2)",' Assuming velocity in column ',ivx
         endif
      endif
-     if (icoltype.gt.0) print "(a,i2)",' Assuming particle type in column ',icoltype
+     if (icoltype.gt.0) then
+        if (ipr==0 .and. iutherm==0 .and. ivx==0) write(*,*)
+        write(*,"(a,i2)") ' Assuming particle type in column ',icoltype
+     endif
+     if ((irho > 0 .or. ih > 0 .or. ipmass > 0) &
+        .and. ipr==0 .and. iutherm==0 .and. ivx==0 .and. icoltype==0) write(*,*)
 
-     if (ndim.eq.0 .or. irho.eq.0 .or. ipmass.eq.0 .or. ih.eq.0) then
-        print "(4(/,a))",' NOTE: Rendering capabilities cannot be enabled', &
-                    '  until positions of density, smoothing length and particle', &
-                    '  mass are known (for the ascii read the simplest way is to ', &
-                    '  label the relevant columns appropriately in the columns file)'
+     if (ndim.gt.0 .and. (irho.eq.0 .or. ipmass.eq.0 .or. ih.eq.0)) then
+        print "(2(/,a))",' NOTE: Rendering disabled until density, h and mass columns known', &
+                    ' (i.e. label relevant columns in file header or columns file)'
      endif
   endif
 
-  if (ivx.gt.0) then
-     iamvec(ivx:ivx+ndimV-1) = ivx
-     labelvec(ivx:ivx+ndimV-1) = 'v'
-     do i=1,ndimV
-       label(ivx+i-1) = 'v\d'//labelcoord(i,1)
-     enddo
-  endif
-  if (iBfirst.gt.0) then
-     iamvec(iBfirst:iBfirst+ndimV-1) = iBfirst
-     labelvec(iBfirst:iBfirst+ndimV-1) = 'B'
-     do i=1,ndimV
-       label(iBfirst+i-1) = 'B\d'//labelcoord(i,1)
-     enddo
-  endif
+  call make_vector_label('v',ivx,ndimV,iamvec,labelvec,label,labelcoord(:,1))
+  call make_vector_label('B',iBfirst,ndimV,iamvec,labelvec,label,labelcoord(:,1))
   !
   !--set labels for each particle type
   !
-  !ntypes = 1 !!maxparttypes
   labeltype(1) = 'gas'
   UseTypeInRenderings(1) = .true.
 
-!-----------------------------------------------------------
-
-  return
 end subroutine set_labels
