@@ -49,7 +49,7 @@
 ! in the module 'particle_data'
 !-------------------------------------------------------------------------
 
-subroutine read_data(rootname,indexstart,nstepsread)
+subroutine read_data(rootname,indexstart,ipos,nstepsread)
   use particle_data
   use params
   use settings_data,  only:ndim,ndimV,ncolumns,ncalc
@@ -57,7 +57,7 @@ subroutine read_data(rootname,indexstart,nstepsread)
   use labels,         only:unitslabel
   use mem_allocation
   implicit none
-  integer, intent(in) :: indexstart
+  integer, intent(in) :: indexstart,ipos
   integer, intent(out) :: nstepsread
   character(len=*), intent(in) :: rootname
   integer, parameter :: maxptmass = 1000
