@@ -48,6 +48,13 @@ program splash
 !
 !     -------------------------------------------------------------------------
 !     Version history/ Changelog:
+!     2.9.1  : (08/11/19)
+!             cleaner menu options for units and calculated quantities;
+!             surface rendering allowed with 3D perspective turned off;
+!             automatic labelling of grain sizes in density and column density plots;
+!             adaptive limits on log colour bars show 3 dex range by default;
+!             auto-adjust limits to device aspect ratio works with multiple panels;
+!             bug fixes with r-z rendering; Toomre Q prompts for mass
 !     2.9.0  : (05/04/19)
 !             general header quantities are read and available in function parser;
 !             more robust label detection and parsing during ascii data read;
@@ -392,13 +399,14 @@ program splash
   use settings_page,      only:interactive,device,nomenu
   use settings_part,      only:initialise_coord_transforms
   use settings_render,    only:icolours,rgbfile
+  use settings_xsecrot,   only:xsec_nomulti
   use colours,            only:rgbtable,ncoltable,icustom
   implicit none
   integer :: i,ierr,nargs,ipickx,ipicky,irender,icontour,ivecplot
   logical :: ihavereadfilenames,evsplash,doconvert,useall,iexist,use_360
   character(len=120) :: string
   character(len=12)  :: convertformat
-  character(len=*), parameter :: version = 'v2.9.0 [5th Apr 2019]'
+  character(len=*), parameter :: version = 'v2.9.1 [8th Nov 2019]'
 
   !
   ! initialise some basic code variables
@@ -696,7 +704,7 @@ program splash
               stop
            endif
            if (irender.gt.0) then
-              if (.not.allowrendering(ipicky,ipickx)) then
+              if (.not.allowrendering(ipicky,ipickx,xsec_nomulti)) then
                  print "(a)",' ERROR: cannot render with x, y choice (must be coords)'
                  stop
               endif
@@ -773,10 +781,10 @@ subroutine print_header
    ' * SPLASH comes with ABSOLUTELY NO WARRANTY.',/, &
    '   This is free software; and you are welcome to redistribute it ',/, &
    '   under certain conditions (see LICENCE file for details). *',/,/, &
-   ' Updates/userguide: http://users.monash.edu.au/~dprice/splash ',/, &
-   ' Email: daniel.price@monash.edu or splash-users@googlegroups.com',/, &
+   ' http://users.monash.edu.au/~dprice/splash ',/, &
+   ' daniel.price@monash.edu or splash-users@googlegroups.com',/, &
    ' Please cite Price (2007), PASA, 24, 159-173 (arXiv:0709.0832) if you ',/, &
-   ' use SPLASH in print and don''t forget to send pics for the gallery.',/)
+   ' use SPLASH in print and don''t forget to send pics for the gallery',/)
 
 end subroutine print_header
 
